/**
 * @brief   GettingStarted/A_ReadOneUncompressedBbeVerse/test.cpp
 * @author  Lynn Allan
 * @version 0.1
 * @date    2003-Nov-27
 * @par     Purpose
 * This is a barebones example for getting started using the CBuilderX
 * package to do software development in The Sword Project. It is intended
 * as one of the very first programs to build. If Matthew 5:3 BBE shows up,
 * the newbie can have some confidence that CBuilderX was installed properly.
 * @par
 * It does not use any sword-api calls in order to simplify project setup. You can
 * ignore most issues related to the setup of include and lib
 *
 * @par     Relationship to Sword-Api code
 * In general, the code mimics how the Sword-Api actually reads a verse from a
 * rawtext file.  The BBE (Bible in Basic English) was used because it has little
 * or no embedded tags. This removes the need to use filters to actually see a
 * verse.
 * @par
 * Variable names are *verbose* variants of actual variables used in Sword-Api code.
 * The convention was to put an underscore ( _ ) after that actual Sword-Api variable
 * name, and then continue with the actual purpose of the variable. For example:
 * "start" becomes "start_OffsetOfVerseInRawtext"
 * This becomes pretty artificial in a hurry, and isn't intended as a recommended
 * practice. The intended 'audience' is a newbie presumed to be pretty disoriented.
 * @par
 * The canon_simplified.h file mimics the actual Sword-Api canon.h and also has some
 * extra comments, some extra arrays to simplify determining the maximum chapter in
 * a book, the maximum verse in a chapter. It leaves out some arrays that are
 * extraneous if parsing isn't used.
 * @par
 * A major difference from actual Sword-Api logic is that parsing of the Bible key
 * isn't done. John 3:16 is 4,3,16. This removes most of VerseKey logic, which keeps
 * a very large amount of dependent code from being required.
 *
 * @par     What is actually happening in the code
 * 1. Open the files
 * 2. Specify the verse to be read (2=NT, 4=John)
 * 3. User canon.h versification to calculate the offset within the vss file
 * 4. Read from the vss file to get the offset and len for the actual rawtext file
 * 5. Read from the rawtext file
 * 6. Display the text from BBE
 *
 * @par     Assumptions
 * BibleCS can be launched and works ok.
 * CBuilderX was installed ok. The sample 'Welcome' HelloWorld can be compiled and executed ok.
 *
 * @par     Disclaimers
 * The coding style used was to enhance clarity and facilitate being able to use the
 * debugger to step through code. *Very* long names were used to convey the purpose
 * of each variable.
 * @par
 * This is *NO* error checking to simplify reading the code.
 * @par
 * stdio functions (sprintf, printf) were used rather than iostream (cout)
 */
#include <stdio.h>
#include <fcntl.h>   // only for _O_RDONLY
#include <io.h>      // for _open, lseek, and read
#include <assert.h>

#include "simplified_canon.h"
#define BUF_MAX 1024

void main()
{
  char  buf_ToHoldVerseRead[BUF_MAX];
  long  offset_WithinRawtextFile_raw;      // magic for versification
  long  offset_WithinRawtextFile_adj;      // magic for versification
  long  index_WithinVersification;         // offset within vss file
  long  start_OffsetOfVerseInRawtext;      // offset within rawtext file
  unsigned short size_OfVerseInBytes;      // len of actual verse
  short  *offset_magic[2][2]  = {{otbks, otcps}, {ntbks, ntcps}};  // this might as well be magic

  int ntFdVss = _open("../../nt.vss", _O_RDONLY);
  int ntFdRawText = _open("../../nt_matthew_partial", _O_RDONLY);
  assert((ntFdVss > 1) && (ntFdRawText > 0));

  printf("Bk  Ch Ver Index    Text\n");
  char testmt = 2;  // OT = 1, NT=2
  int book    = 1;  // OT-1 = Genesis, NT-1=Matthew
  int chap    = 5;
  int verse   = 3; // NT Matthew 5:3 (first verse of Beatitudes from Sermon on the Mount
  //Following is simplification of magic that happens in VerseKey::Index
  offset_WithinRawtextFile_raw = offset_magic[testmt-1][0][book];
  offset_WithinRawtextFile_adj = offset_magic[testmt-1][1][(int)offset_WithinRawtextFile_raw + chap];
  index_WithinVersification = (offset_WithinRawtextFile_adj + verse) * 6;
  //Following is simplification of what happens in RawVerse::findOffset
  lseek(ntFdVss, index_WithinVersification, SEEK_SET);
  //Following is simplification of what happens in RawVerse::readText
  read(ntFdVss, &start_OffsetOfVerseInRawtext, 4);
  read(ntFdVss, &size_OfVerseInBytes, 2);
  lseek(ntFdRawText, start_OffsetOfVerseInRawtext, SEEK_SET);
  read(ntFdRawText, buf_ToHoldVerseRead, (int)size_OfVerseInBytes);
  buf_ToHoldVerseRead[size_OfVerseInBytes] = 0;  // Just to make sure text null terminated
  printf("%2d %3d %3d %08d %s\n", book, chap, verse, index_WithinVersification, buf_ToHoldVerseRead);
}
